@@ -62,7 +62,7 @@ extern Tcl_Interp *xcinterp;
 #endif
 
 extern Display    *dpy;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern ApplicationData appdata;
 extern int 	  *appcolors;
 extern short	  popups;     /* total number of popup windows */
@@ -105,7 +105,7 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
    if (filenum == flcurrent) return;
 
    if (flcurrent >= 0) 		/* erase previous box */
-      XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+      XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
@@ -113,7 +113,7 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
 
    twidth = XTextWidth(appdata.filefont, files[filenum].filename,
 	    strlen(files[filenum].filename));
-   XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+   XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (filenum
 	   - flstart), twidth + 10, FILECHARHEIGHT);
 
@@ -134,17 +134,17 @@ void startfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
    xcAddEventHandler(w, PointerMotionMask, False, (xcEventHandler)dragfilebox, NULL);
 #endif
 
-   XSetFunction(dpy, areastruct.gc, GXcopy);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR);
-   XSetLineAttributes(dpy, areastruct.gc, 1, LineSolid, CapRound, JoinMiter);
+   XSetFunction(dpy, areawin->gc, GXcopy);
+   XSetForeground(dpy, areawin->gc, AUXCOLOR);
+   XSetLineAttributes(dpy, areawin->gc, 1, LineSolid, CapRound, JoinMiter);
 
    /* draw initial box */
 
    flcurrent = -1;
    dragfilebox(w, NULL, (XMotionEvent *)event);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   XSetFunction(dpy, areawin->gc, GXxor);
+   XSetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -155,7 +155,7 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 {
    Window lwin = xcWindow(w);
 
-   XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+   XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
@@ -168,8 +168,8 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 #endif
 
    /* Restore graphics state values */
-   XSetForeground(dpy, areastruct.gc, areastruct.gccolor);
-   XSetFunction(dpy, areastruct.gc, areastruct.gctype);
+   XSetForeground(dpy, areawin->gc, areawin->gccolor);
+   XSetFunction(dpy, areawin->gc, areawin->gctype);
 }
 
 /*----------------------------------------------------------------------*/
@@ -202,6 +202,7 @@ char *getcrashfilename()
 	    break;
 	 }
       }
+      fclose(fi);
    }
    return retstr;
 }
@@ -232,13 +233,16 @@ void findcrashfiles()
    struct stat sbuf;
    uid_t userid = getuid();
    time_t recent = 0;
+   char *snptr;
+   int pid;
 
    cwd = opendir(xobjs.tempdir);
    if (cwd == NULL) return;	/* No tmp directory, no tmp files! */
 
    while ((dp = readdir(cwd)) != NULL) {
       sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
-      if (!strncmp(_STR + strlen(xobjs.tempdir) + 1, "XC", 2)) {
+      snptr = _STR + strlen(xobjs.tempdir) + 1;
+      if (!strncmp(snptr, "XC", 2)) {
          if ((!stat(_STR, &sbuf)) && (sbuf.st_uid == userid)) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
 	       recent = sbuf.st_ctime;
@@ -258,12 +262,14 @@ void findcrashfiles()
 
 #ifdef TCL_WRAPPER
       char *cfile = getcrashfilename();
+
       sprintf(_STR, ".query.title.field configure -text "
 		"\"Recover file \'%s\'?\"", 
 		(cfile == NULL) ? "(unknown)" : cfile);
       Tcl_Eval(xcinterp, _STR);
       Tcl_Eval(xcinterp, ".query.bbar.okay configure -command "
 		"{filerecover; wm withdraw .query}; wm deiconify .query");
+      if (cfile != NULL) free(cfile);
 #else
       getfile(NULL, (pointertype)RECOVER, NULL);   /* Crash recovery mode */
 #endif
@@ -370,7 +376,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 	    else if (match_filter(dp->d_name, filter))
 	       files[flfiles].filetype = MATCH;
 	    else {
-	       if (areastruct.filefilter)
+	       if (areawin->filefilter)
 		  continue;
 	       else
 	          files[flfiles].filetype = NONMATCH;
@@ -399,7 +405,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
       pixheight = flfiles * FILECHARHEIGHT + 25;
       if (pixheight < textheight) pixheight = textheight;
 
-      flistpix = XCreatePixmap(dpy, areastruct.areawin, textwidth, pixheight,
+      flistpix = XCreatePixmap(dpy, areawin->window, textwidth, pixheight,
 	   DefaultDepthOfScreen(xcScreen(w)));
 
       /* Write the filenames onto the pixmap */
@@ -445,7 +451,7 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
 #ifdef TCL_WRAPPER
    int bval;
    int result;
-   char *cstr = Tcl_GetVar2(xcinterp, "XCOps", "filter", TCL_NAMESPACE_ONLY);
+   char *cstr = (char *)Tcl_GetVar2(xcinterp, "XCOps", "filter", TCL_NAMESPACE_ONLY);
    if (cstr == NULL) {
       Wprintf("Error: No variable $XCOps(filter) in Tcl!");
       return;
@@ -455,7 +461,7 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
       Wprintf("Error: Bad variable $XCOps(filter) in Tcl!");
       return;  
    }
-   areastruct.filefilter = bval;
+   areawin->filefilter = bval;
 #endif
 
    for (n = 0; n < flfiles; n++)
@@ -680,7 +686,7 @@ void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
 void setfilefilter(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
-   areastruct.filefilter = (areastruct.filefilter) ? False : True;
+   areawin->filefilter = (areawin->filefilter) ? False : True;
 
    /* Force regeneration of the file list */
 
@@ -770,7 +776,7 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
    wwidth = XTextWidth(appdata.xcfont, "filter", strlen("filter"));
    XtnSetArg(XtNx, width - wwidth - 50);
    XtnSetArg(XtNy, LISTHEIGHT + 10);
-   XtnSetArg(XtNset, areastruct.filefilter);
+   XtnSetArg(XtNset, areawin->filefilter);
    XtnSetArg(XtNsquare, True);
    XtnSetArg(XtNborderWidth, 0);
    XtnSetArg(XtNfont, appdata.xcfont);
@@ -792,11 +798,11 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 /* Look for a directory name in a string and update cwdname accordingly	   */
 /*-------------------------------------------------------------------------*/
 
-int lookdirectory(char *lstring)
+int lookdirectory(char *lstring, int nchars)
 {
    int slen;
 
-   xc_tilde_expand(lstring);
+   xc_tilde_expand(lstring, nchars);
    slen = strlen(lstring);
 
    if (lstring[slen - 1] == '/' || (opendir(lstring) != NULL)) {
