@@ -128,15 +128,16 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* Names used for convenience throughout the source code */
 
-#define topobject	areastruct.topinstance->thisobject
-#define hierobject	areastruct.hierstack->thisinst->thisobject
+#define eventmode	areawin->event_mode
+#define topobject	areawin->topinstance->thisobject
+#define hierobject	areawin->hierstack->thisinst->thisobject
 
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
 
 #define ENDPART		topobject->plist + topobject->parts
-#define EDITPART	topobject->plist + areastruct.editpart
+#define EDITPART	topobject->plist + areawin->editpart
 
 /* Type checks (argument "a" is type genericptr) */
 
@@ -165,7 +166,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 /* conversions from a selection to a specific type */
 
-#define SELTOGENERICPTR(a) ((areastruct.hierstack == NULL) ? \
+#define SELTOGENERICPTR(a) ((areawin->hierstack == NULL) ? \
 			(topobject->plist + *(a)) : \
 			(hierobject->plist + *(a)))
 
@@ -495,7 +496,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define WAITFOR		appcursors[9]
 #define HAND		appcursors[10]
 
-#define DEFAULTCURSOR	(*areastruct.defaultcursor)
+#define DEFAULTCURSOR	(*areawin->defaultcursor)
 
 /*----------------------------------------------------------------------*/
 /* selection-mechanism structures					*/
@@ -776,7 +777,7 @@ typedef struct _selection {
 #define select_element(a) recurse_select_element(a, UNDO_MORE) 
 #define select_add_element(a) recurse_select_element(a, UNDO_DONE) 
 
-#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areawin->topinstance)
 
 /*----------------------------------------------------------------------*/
 /* Netlist structures for schematic capture				*/
@@ -944,7 +945,7 @@ typedef struct _matrix {
 /* Some convenience functions for matrix manipulation			*/
 /*----------------------------------------------------------------------*/
 
-#define DCTM  areastruct.MatStack
+#define DCTM  areawin->MatStack
 
 /*----------------------------------------------------------------------*/
 /* button tap/press definitions 					*/
@@ -1091,19 +1092,19 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 #define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areastruct.gc, FOREGROUND); else SetForeground(dpy, areastruct.gc, a)
+	areawin->gc, FOREGROUND); else SetForeground(dpy, areawin->gc, a)
 
 #define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
-	areastruct.gc, FOREGROUND ^ b); else SetForeground(dpy, areastruct.gc,\
+	areawin->gc, FOREGROUND ^ b); else SetForeground(dpy, areawin->gc,\
 	a ^ b)
 
-#define XcSetFunction(z) SetFunction(dpy, areastruct.gc, z); \
-	areastruct.gctype = z
+#define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
+	areawin->gctype = z
 
-#define XcSetForeground(z) XTopSetForeground(z); areastruct.gccolor = \
+#define XcSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
-#define XcSetXORFg(y,z) XSetXORFg(y,z); areastruct.gccolor = \
+#define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
 /*----------------------------------------------------------------------*/
@@ -1265,7 +1266,7 @@ typedef struct {
    /* widgets and X11 parameters */
    xcWidget	area;
    xcWidget	scrollbarh, scrollbarv;
-   Window	areawin;
+   Window	window;
    GC 		gc;
    int		gccolor, gctype;
    xcIntervalId time_id;
@@ -1299,6 +1300,7 @@ typedef struct {
    Boolean	editinplace;
    Boolean	schemon;
    Boolean	pinpointon;
+   Boolean	attachto;
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
 #endif
@@ -1313,11 +1315,12 @@ typedef struct {
    Matrixptr	MatStack;
    pushlistptr	stack;
    pushlistptr	hierstack;
+   short	event_mode;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
-} Clientdata;
+} XCWindowData;
 
 /* Record for undo function */ 
 
@@ -1352,6 +1355,8 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
+   char		*libsearchpath;	 /* list of directories to search */
+   char		*filesearchpath; /* list of directories to search */
    char		*tempfile;
    char		*tempdir;
    Boolean	retain_backup;
@@ -1369,6 +1374,8 @@ typedef struct {
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
+   XCWindowData *windowlist;
+   short	windows;
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
